/**

	This is an example of the use of the Container class library. It contains some
	benchmark comparisons with the JAVA development kit (jdk) Vector class from Sun.

*/

// The global variables and functions declared 
// in this file will be stored in this class
namespace Demo;

// Allows the use of all static variables and methods of System
// without extra qualification through the classname. Ambigiouties 
// will be checked.
using namespace java.lang.System;

import jump.basics.*;
import java.io.*;
import java.util.Random;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;

class Benchmark
{
	String name;
	long start, total;
	int n, cycle;
public:
	Benchmark( String Name, int n = 10 ) { name = Name; cycle = n; }
	void clear( String newname ) { start = total = 0; n = 0; name = newname; }
	void start() { start = (new Date()).getTime(); }
	void stop() {
		total += (new Date()).getTime() - start;
		n++;
		if( cycle > 0 && n % cycle == 0 ) print(); }
	void cmp( Benchmark to ) {
		if( total == 0 ) out.println( name + " : stopped time too short" );
		else if( to.total == 0 ) out.println( to.name + " : stopped time too short" );
		else {
			int percent = (int)((total * 100) / to.total);
			if( percent > 100 ) 
				out.println( name + " requires " + percent + " % of the time of " + to.name + "\n" );
			else 
				out.println( to.name + " requires " + (to.total * 100) / total + " % of the time of " + name + "\n" ); } }
	void print() { out.println( "Benchmark: " + name + " x " + n + " required " + total + " ms." ); }
}

/*
	CompString is used to compare 2 Strings
*/
class CompString extends BinPredicate
{
	public int exec( Object o1, Object o2 ) { return o1.toString().compareTo( o2.toString() ); }
}

/*
	SumDouble is used to calculate the sum of all elements of a container which contains doubles
*/
class SumDouble extends UnPredicate
{
	double sum;
	public int exec( double d ) { sum += d; return 0; }
}

class StreamOut extends UnPredicate
{
	DataOutputStream os;
	StreamOut( OutputStream s ) { os = new DataOutputStream( s ); }
	public int exec( Object o ) { os.writeBytes( o.toString() ); os.writeByte( '\n' ); return 0; }
}

class StreamIn extends UnPredicate
{
	DataInputStream os;
	StreamIn( InputStream s ) { os = new DataInputStream( s ); }
	public int exec( Object o ) { o = new String( os.readLine() ); return 0; }
}

static Random randomizer = new Random();

int makeRandomInt( int max )
{
	return Math.abs( randomizer.nextInt() % max );
}

String makeRandomString()
{
	int len = makeRandomInt( 30 ) + 1;
	StringBuffer buffer = new StringBuffer( len );
	for( int i = 0; i < len; i++ )
		buffer.append( (char) ( makeRandomInt( 25 ) + 'A' ) );
	return buffer.toString();
}

String[] makeRandomStringArray( int size )
{
	String as[] = new String[ size ];
	for( int i = size; i-- > 0; as[ i ] = makeRandomString() );
	return as;
}

void main( String argv[] )
{
	int size = 10000;

	Benchmark jdkBench = new Benchmark( "jdk Vector adding " + size + " Strings", 1 );
	Benchmark jumpBench = new Benchmark( "jump Set adding " + size + " Strings", 1 );

	// Generating an array of random Strings
	String S = "test", as[] = makeRandomStringArray( size );
	out.println( "Ready creating " + size + " Strings" );

	// Comparing add
	jdkBench.start();
	Vector jdkVector = new Vector( size );
	for( int j = size; j-- > 0; ) jdkVector.addElement( as[ j ] );
	jdkBench.stop();

	jumpBench.start();
	JumpSet< String > jumpStringSet = new JumpSet< String >( size );
	for( int j = size; j-- > 0;  ) jumpStringSet.add( as[ j ] );
	jumpBench.stop();

	jumpBench.cmp( jdkBench );

	int itercnt = 100;

	// Comparing iterate
	jdkBench.clear( "jdk Vector iterating " + itercnt * size + " times" );
	jumpBench.clear( "jump Set iterating " + itercnt * size + " times"  );

	jdkBench.start();
	for( int i = itercnt; i-- > 0; )
		for( int j = size; j-- > 0;  ) S = (String)jdkVector.elementAt( j );
	jdkBench.stop();

	jumpBench.start();
	for( int i = itercnt; i-- > 0; )
		for( int j = size; j-- > 0;  ) S = jumpStringSet[ j ];
	jumpBench.stop();

	jumpBench.cmp( jdkBench );

	// Comparing insert
	jdkBench.clear( "jdk Vector inserting " + size + " Strings" );
	jumpBench.clear( "jump Set inserting " + size + " Strings" );

	jdkBench.start();
	for( int j = size; j-- > 0;  ) jdkVector.insertElementAt( S, j );
	jdkBench.stop();

	jumpBench.start();
	for( int j = size; j-- > 0;  ) jumpStringSet.insAt( j, S );
	jumpBench.stop();

	jumpBench.cmp( jdkBench );

	// Comparing remove
	jdkBench.clear( "jdk Vector removing " + size + " Strings" );
	jumpBench.clear( "jump Set removing " + size + " Strings" );

	jdkBench.start();
	for( int j = size; j-- > 0;  ) jdkVector.removeElementAt( j );
	jdkBench.stop();

	jumpBench.start();
	for( int j = size; j-- > 0;  ) jumpStringSet.rmvAt( j );
	jumpBench.stop();

	jumpBench.cmp( jdkBench );

	Benchmark jumpSBench = new Benchmark( "jump Set sorted insert of " + size + " Strings", 1 );
	jumpBench.clear( "jump Set sorting " + size + " Strings" );

	jumpSBench.start();
	JumpSortedSet< String > jumpSortedSet = new JumpSortedSet< String >( new CompString() );
	for( int j = size; j-- > 0;  ) jumpSortedSet.add( as[ j ] );
	jumpSBench.stop();

	jumpStringSet.clear();

	jumpBench.start();
	for( int j = size; j-- > 0; ) jumpStringSet.add( as[ j ]);
	jumpStringSet.sort( new CompString() );
	jumpBench.stop();

	jumpBench.cmp( jumpSBench );

	// Verifying
	for( int j = size; j-- > 0;  )
		if( jumpSortedSet[ j ] .compareTo( jumpStringSet[ j ] ) != 0 )
			System.out.println( j + ": " +jumpStringSet[ j] + " != " + jumpSortedSet[ j ] );
	
	// Writing the contents of the set to a file
	jumpStringSet.forEach( new StreamOut( new FileOutputStream( "set.out" ) ) );

	// Using a set with a native type
	JumpSet< double > ad = new JumpSet< double >(size);

	// Filling with values
	for( int j = size; j-- > 0; ) ad.add( makeRandomInt( 1000 ) );

	// Using forEach to calculate sum
	SumDouble total = new SumDouble();
	ad.forEach( total );
	out.println( "Total sum is " + total.sum );

	// Sorting
	ad.sort( new BinPredicate() );

	// Verifying sort result
	out.println( "Verifying sort result." );
	int k = 0;
	for( int j = size; j-- > 1; )
		if( ad[j] < ad[j-1] ) {
			out.println( "Error " + k++ + " at " + ad[j] +" " + ad[ j-1 ] ); }
	if( k == 0 ) out.println( "No error !" );

	// Recalculating sum
	total.sum = 0;
	ad.forEach( total );
	out.println( "Total sum is " + total.sum );

	double sum = 0;
	//	Iterating over  all elements to calculate sum
	SetIterator< double > diter = new SetIterator< double >( ad );
	while( diter.hasMoreElements() ) 
		sum += diter++;
	out.println( "Total sum is " + sum );
}
