package jump.basics;

/**

	A JumpSortedSet is a generic container that provides very fast associative lookup 
	as well as the possibility to use it as a sorted container with any desired sort order.

	The implementation provides high performance typesafe containers with very	little 
	overhead since the algorithms for searching and sorting are shared.

	When creating a new container you can provide a boolean to check for uniqueness when
	adding new elements, and an integer, which is the delta used to increase the size 
	when the Container needs to grow.

	Copyright ( c ) 1996 Detlef Hoeffner

*/

template < class T > public class JumpSortedSet extends JumpSet< T >
{
	BinPredicate comparator;
	boolean bUnique;
public:
	JumpSortedSet() { super( 100 ); setComparator( null ); }
	JumpSortedSet( BinPredicate comp, boolean unique = false, int d = 100 ) {
		super( d ); bUnique = unique; setComparator( comp ); }
	JumpSortedSet( JumpSortedSet s ) { super( s ); comparator = s.comparator; bUnique = s.bUnique; }
	JumpSortedSet( T[] array, BinPredicate comp, int d = 100 ) {
		super( array, d ); setComparator( comp ); sort( comp ); }
	final void setComparator( BinPredicate comp ) { comparator = comp == null ? default_comparator : comp; }
	Object clone() { return new JumpSortedSet( this ); }
	int add( T t ) {
		int idx = Sort.findLessEql( t, data, cur, comparator );
		if( idx >= 0 ) {
			if( bUnique && comparator.exec( t, data[ idx ] ) == 0 ) return -1; }
		else idx = cur;
		insAt( idx, t );
		return idx; }
	int findLessEql( T t ) { return Sort.findLessEql( t, data, cur, comparator ); }
	int find( T t ) {
		int idx = Sort.findLessEql( t, data, cur, comparator );
		return idx >= 0 && comparator.exec( t, data[ idx ] ) == 0 ? idx : -1; }
	int findNext( int idx ) {
		return ++idx < cur && comparator.exec( data[ idx - 1 ], data[ idx ] ) == 0 ? idx : -1; }
}
