package jump.basics;

/**

	Sort contains high performance sorting and searching algorithms for native and reference types.

	Copyright ( c ) 1996 Detlef Hoeffner

*/

final public class Sort
{
	static int findLessEql( byte t, byte data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( t <= data[ mid ] ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static int findLessEql( char t, char data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( t <= data[ mid ] ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static int findLessEql( short t, short data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( t <= data[ mid ] ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static int findLessEql( int t, int data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( t <= data[ mid ] ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static int findLessEql( long t, long data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( t <= data[ mid ] ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static int findLessEql( float t, float data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( t <= data[ mid ] ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static int findLessEql( double t, double data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( t <= data[ mid ] ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static int findLessEql( Object t, Object data[], int max, BinPredicate comparator ) {
		int low = 0, high = max;
		while( low < high ) {
			int mid = (high + low) / 2;
			if( comparator.exec( t, data[ mid ] ) <= 0 ) high = mid;
			else low = mid + 1; }
		return low < max ? low : -1; }
	static void quickSort( byte data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		byte v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( data[ ++i ] < v );
			while( --j > 0 && data[ j ] > v );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
	static void quickSort( short data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		short v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( data[ ++i ] < v );
			while( --j > 0 && data[ j ] > v );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
	static void quickSort( char data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		char v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( data[ ++i ] < v );
			while( --j > 0 && data[ j ] > v );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
	static void quickSort( int data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		int v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( data[ ++i ] < v );
			while( --j > 0 && data[ j ] > v );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
	static void quickSort( long data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		long v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( data[ ++i ] < v );
			while( --j > 0 && data[ j ] > v );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
	static void quickSort( float data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		float v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( data[ ++i ] < v );
			while( --j > 0 && data[ j ] > v );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
	static void quickSort( double data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		double v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( data[ ++i ] < v );
			while( --j > 0 && data[ j ] > v );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
	static void quickSort( Object data[], int l, int r, BinPredicate comp ) {
		if( r <= l ) return;
		Object v = data[ r ], tmp;
		int i = l - 1,	j = r;
		do {
			while( comp.exec( data[ ++i ], v ) < 0 );
			while( --j > 0 && comp.exec( data[ j ], v ) > 0 );
			tmp = data[ i ];
			data[ i ] = data[ j ];
			data[ j ] = tmp;
		} while( j > i );
		data[ j ] = data[ i ];
		data[ i ] = data[ r ];
		data[ r ] = tmp;
		quickSort( data, l, i - 1, comp );
		quickSort( data, i + 1, r, comp ); }
}
